package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.TooLowQuantityException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.dto.InventoryDto;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class InventoryService {

    private MaterialService materialService;

    public InventoryService(MaterialService materialService) {
        this.materialService = materialService;
    }

    /**
     * Utilise tous les produits demandés.
     *
     * @param mixes Tous les produits à utiliser
     * @throws TooLowQuantityException Lorsqu'il n'y a pas assez d'un produit dans l'inventaire
     */
    @Transactional
    public void use(List<InventoryDto> mixes) throws TooLowQuantityException {
        for (InventoryDto mix : mixes) {
            List<Material> materials = convertMaterialIdsToMaterials(mix.getMaterials());

            for (int i = 0; i < materials.size(); i++) {
                Material material = materials.get(i);
                Float quantity = mix.getQuantities().get(i);

                if (!hasEnoughStock(material, quantity))
                    throw new TooLowQuantityException(mix.getMixId(), material, material.getInventoryQuantity(), quantity);

                useMaterial(material, quantity);
            }
        }
    }

    /**
     * Utilise un produit.
     *
     * @param material Le produit à utiliser
     * @param quantity La quantité à utiliser
     */
    private void useMaterial(Material material, Float quantity) {
        material.setInventoryQuantity(material.getInventoryQuantity() - quantity);
        materialService.update(material);
    }

    /**
     * Vérifie s'il y a assez d'un produit dans l'inventaire pour l'utiliser.
     *
     * @param material Le produit à vérifier
     * @param quantity La quantité à utiliser
     * @return S'il y a assez de produit dans l'inventaire pour l'utiliser
     */
    private boolean hasEnoughStock(Material material, Float quantity) {
        return material.getInventoryQuantity() >= quantity;
    }

    /**
     * Convertit une liste d'identifiant de produit en produits.
     *
     * @param ids Les identifiant à convertir
     * @return Les produits correspondant aux identifiants
     */
    private List<Material> convertMaterialIdsToMaterials(List<Long> ids) {
        return ids.stream()
                .map(i -> materialService.getById(i))
                .collect(Collectors.toList());
    }
}
